@@ -7,6 +7,8 @@ Released under MIT license, see LICENSE file
 """
 import os
 import sys
+import io
+import tempfile
 
 import mock
 import pytest
@@ -22,7 +24,7 @@ from conftest import run_cmd, normalize, BASE_HELP, HELP_HISTORY, SHORTCUTS_TXT,
 
 
 def test_ver():
-    assert cmd2.__version__ == '0.7.4'
+    assert cmd2.__version__ == '0.7.7'
 
 
 def test_empty_statement(base_app):
@@ -99,7 +101,7 @@ To enable full traceback, run the following command:  'set debug true'
 """)
     assert normalize(str(err)) == expected
 
-def test_set_abbreviated(base_app):
+def test_set_quiet(base_app):
     out = run_cmd(base_app, 'set quie True')
     expected = normalize("""
 quiet - was: False
@@ -113,7 +115,10 @@ now: True
 
 def test_base_shell(base_app, monkeypatch):
     m = mock.Mock()
-    monkeypatch.setattr("subprocess.Popen", m)
+    subprocess = 'subprocess'
+    if six.PY2:
+        subprocess = 'subprocess32'
+    monkeypatch.setattr("{}.Popen".format(subprocess), m)
     out = run_cmd(base_app, 'shell echo a')
     assert out == []
     assert m.called
@@ -317,19 +322,17 @@ def test_base_load(base_app, request):
     test_dir = os.path.dirname(request.module.__file__)
     filename = os.path.join(test_dir, 'script.txt')
 
-    # The way the load command works, we can't directly capture its stdout or stderr
+    assert base_app.cmdqueue == []
+    assert base_app._script_dir == []
+    assert base_app._current_script_dir is None
+
+    # Run the load command, which populates the command queue and sets the script directory
     run_cmd(base_app, 'load {}'.format(filename))
 
-    # But what we can do is check the history to see what commands have been run ...
-    out = run_cmd(base_app, 'history')
-
-    # TODO: Figure out why when we unit test the command this way the commands from the script aren't shown in history
-    # NOTE: It works correctly when we run it at the command line
-    expected = normalize("""
--------------------------[1]
-load {}
-""".format(filename))
-    assert out == expected
+    assert base_app.cmdqueue == ['help history', 'eos']
+    sdir = os.path.dirname(filename)
+    assert base_app._script_dir == [sdir]
+    assert base_app._current_script_dir == sdir
 
 def test_load_with_empty_args(base_app, capsys):
     # The way the load command works, we can't directly capture its stdout or stderr
@@ -339,6 +342,7 @@ def test_load_with_empty_args(base_app, capsys):
     # The load command requires a file path argument, so we should get an error message
     expected = normalize("""ERROR: load command requires a file path:\n""")
     assert normalize(str(err)) == expected
+    assert base_app.cmdqueue == []
 
 
 def test_load_with_nonexistent_file(base_app, capsys):
@@ -349,6 +353,7 @@ def test_load_with_nonexistent_file(base_app, capsys):
     # The load command requires a path to an existing file
     assert str(err).startswith("ERROR")
     assert "does not exist or is not a file" in str(err)
+    assert base_app.cmdqueue == []
 
 
 def test_load_with_empty_file(base_app, capsys, request):
@@ -362,6 +367,7 @@ def test_load_with_empty_file(base_app, capsys, request):
     # The load command requires non-empty scripts files
     assert str(err).startswith("ERROR")
     assert "is empty" in str(err)
+    assert base_app.cmdqueue == []
 
 
 def test_load_with_binary_file(base_app, capsys, request):
@@ -375,48 +381,54 @@ def test_load_with_binary_file(base_app, capsys, request):
     # The load command requires non-empty scripts files
     assert str(err).startswith("ERROR")
     assert "is not an ASCII or UTF-8 encoded text file" in str(err)
+    assert base_app.cmdqueue == []
 
 
 def test_load_with_utf8_file(base_app, capsys, request):
     test_dir = os.path.dirname(request.module.__file__)
     filename = os.path.join(test_dir, 'scripts', 'utf8.txt')
 
-    # The way the load command works, we can't directly capture its stdout or stderr
-    run_cmd(base_app, 'load {}'.format(filename))
-    out, err = capsys.readouterr()
+    assert base_app.cmdqueue == []
+    assert base_app._script_dir == []
+    assert base_app._current_script_dir is None
 
-    # TODO Make this test better once shell command is fixed to used cmd2's stdout
-    assert str(err) == ''
+    # Run the load command, which populates the command queue and sets the script directory
+    run_cmd(base_app, 'load {}'.format(filename))
+
+    assert base_app.cmdqueue == ['!echo γνωρίζω', 'eos']
+    sdir = os.path.dirname(filename)
+    assert base_app._script_dir == [sdir]
+    assert base_app._current_script_dir == sdir
 
 
 def test_base_relative_load(base_app, request):
     test_dir = os.path.dirname(request.module.__file__)
     filename = os.path.join(test_dir, 'script.txt')
 
-    # The way the load command works, we can't directly capture its stdout or stderr
+    assert base_app.cmdqueue == []
+    assert base_app._script_dir == []
+    assert base_app._current_script_dir is None
+
+    # Run the load command, which populates the command queue and sets the script directory
     run_cmd(base_app, '_relative_load {}'.format(filename))
 
-    # But what we can do is check the history to see what commands have been run ...
-    out = run_cmd(base_app, 'history')
-
-    # TODO: Figure out why when we unit test the command this way the commands from the script aren't shown in history
-    # NOTE: It works correctly when we run it at the command line
-    expected = normalize("""
--------------------------[1]
-_relative_load {}
-""".format(filename))
-    assert out == expected
+    assert base_app.cmdqueue == ['help history', 'eos']
+    sdir = os.path.dirname(filename)
+    assert base_app._script_dir == [sdir]
+    assert base_app._current_script_dir == sdir
 
 def test_relative_load_requires_an_argument(base_app, capsys):
     run_cmd(base_app, '_relative_load')
     out, err = capsys.readouterr()
     assert out == ''
     assert err.startswith('ERROR: _relative_load command requires a file path:\n')
+    assert base_app.cmdqueue == []
 
 
 def test_base_save(base_app):
     # TODO: Use a temporary directory for the file
     filename = 'deleteme.txt'
+    base_app.feedback_to_output = True
     run_cmd(base_app, 'help')
     run_cmd(base_app, 'help save')
 
@@ -462,6 +474,7 @@ def test_save_parse_error(base_app, capsys):
 
 def test_save_tempfile(base_app):
     # Just run help to make sure there is something in the history
+    base_app.feedback_to_output = True
     run_cmd(base_app, 'help')
     out = run_cmd(base_app, 'save *')
     output = out[0]
@@ -495,25 +508,63 @@ def test_save_invalid_path(base_app, capsys):
 
 
 def test_output_redirection(base_app):
-    # TODO: Use a temporary directory/file for this file
-    filename = 'out.txt'
+    fd, filename = tempfile.mkstemp(prefix='cmd2_test', suffix='.txt')
+    os.close(fd)
 
-    # Verify that writing to a file works
-    run_cmd(base_app, 'help > {}'.format(filename))
-    expected = normalize(BASE_HELP)
-    with open(filename) as f:
-        content = normalize(f.read())
-    assert content == expected
+    try:
+        # Verify that writing to a file works
+        run_cmd(base_app, 'help > {}'.format(filename))
+        expected = normalize(BASE_HELP)
+        with open(filename) as f:
+            content = normalize(f.read())
+        assert content == expected
 
-    # Verify that appending to a file also works
-    run_cmd(base_app, 'help history >> {}'.format(filename))
-    expected = normalize(BASE_HELP + '\n' + HELP_HISTORY)
-    with open(filename) as f:
-        content = normalize(f.read())
-    assert content == expected
+        # Verify that appending to a file also works
+        run_cmd(base_app, 'help history >> {}'.format(filename))
+        expected = normalize(BASE_HELP + '\n' + HELP_HISTORY)
+        with open(filename) as f:
+            content = normalize(f.read())
+        assert content == expected
+    except:
+        raise
+    finally:
+        os.remove(filename)
 
-    # Delete file that was created
-    os.remove(filename)
+
+def test_feedback_to_output_true(base_app):
+    base_app.feedback_to_output = True
+    base_app.timing = True
+    f, filename = tempfile.mkstemp(prefix='cmd2_test', suffix='.txt')
+    os.close(f)
+
+    try:
+        run_cmd(base_app, 'help > {}'.format(filename))
+        with open(filename) as f:
+            content = f.readlines()
+        assert content[-1].startswith('Elapsed: ')
+    except:
+        raise
+    finally:
+        os.remove(filename)
+
+
+def test_feedback_to_output_false(base_app, capsys):
+    base_app.feedback_to_output = False
+    base_app.timing = True
+    f, filename = tempfile.mkstemp(prefix='feedback_to_output', suffix='.txt')
+    os.close(f)
+
+    try:
+        run_cmd(base_app, 'help > {}'.format(filename))
+        out, err = capsys.readouterr()
+        with open(filename) as f:
+            content = f.readlines()
+        assert not content[-1].startswith('Elapsed: ')
+        assert err.startswith('Elapsed')
+    except:
+        raise
+    finally:
+        os.remove(filename)
 
 
 def test_allow_redirection(base_app):
@@ -537,40 +588,59 @@ def test_input_redirection(base_app, request):
 
     # NOTE: File 'redirect.txt" contains 1 word "history"
 
-    # Verify that redirecting input from a file works
+    # Verify that redirecting input ffom a file works
     out = run_cmd(base_app, 'help < {}'.format(filename))
     expected = normalize(HELP_HISTORY)
     assert out == expected
 
 
-def test_pipe_to_shell(base_app):
+def test_pipe_to_shell(base_app, capsys):
     if sys.platform == "win32":
         # Windows
+        command = 'help | sort'
         # Get help menu and pipe it's output to the sort shell command
-        out = run_cmd(base_app, 'help | sort')
-        expected = ['', '', '_relative_load  edit  history  py        quit  save  shell      show',
-                    '========================================',
-                    'cmdenvironment  help  load     pyscript  run   set   shortcuts',
-                    'Documented commands (type help <topic>):']
-        assert out == expected
+        # expected = ['', '', '_relative_load  edit  history  py        quit  save  shell      show',
+        #             '========================================',
+        #             'cmdenvironment  help  load     pyscript  run   set   shortcuts',
+        #             'Documented commands (type help <topic>):']
+        # assert out == expected
     else:
         # Mac and Linux
         # Get help on help and pipe it's output to the input of the word count shell command
-        out = run_cmd(base_app, 'help help | wc')
+        command = 'help help | wc'
+        # # Mac and Linux wc behave the same when piped from shell, but differently when piped stdin from file directly
+        # if sys.platform == 'darwin':
+        #     expected = "1      11      70"
+        # else:
+        #     expected = "1 11 70"
+        # assert out.strip() == expected.strip()
 
-        # Mac and Linux wc behave the same when piped from shell, but differently when piped stdin from file directly
-        if sys.platform == 'darwin':
-            expected = normalize("1      11      70")
+    run_cmd(base_app, command)
+    out, err = capsys.readouterr()
+
+    # Unfortunately with the improved way of piping output to a subprocess, there isn't any good way of getting
+    # access to the output produced by that subprocess within a unit test, but we can verify that no error occured
+    assert not err
+
+def test_pipe_to_shell_error(base_app, capsys):
+    # Try to pipe command output to a shell command that doesn't exist in order to produce an error
+    run_cmd(base_app, 'help | foobarbaz.this_does_not_exist')
+    out, err = capsys.readouterr()
+
+    assert not out
+
+    expected_error = 'FileNotFoundError'
+    if six.PY2:
+        if sys.platform.startswith('win'):
+            expected_error = 'WindowsError'
         else:
-            expected = normalize("1 11 70")
-        assert out[0].strip() == expected[0].strip()
+            expected_error = 'OSError'
+    assert err.startswith("EXCEPTION of type '{}' occurred with message:".format(expected_error))
 
 
 @pytest.mark.skipif(not cmd2.can_clip,
                     reason="Pyperclip could not find a copy/paste mechanism for your system")
 def test_send_to_paste_buffer(base_app):
-    from cmd2 import can_clip
-
     # Test writing to the PasteBuffer/Clipboard
     run_cmd(base_app, 'help >')
     expected = normalize(BASE_HELP)
@@ -591,9 +661,9 @@ now: True
     assert out == expected
     out, err = capsys.readouterr()
     if sys.platform == 'win32':
-        assert out.startswith('Elapsed: 0:00:00')
+        assert err.startswith('Elapsed: 0:00:00')
     else:
-        assert out.startswith('Elapsed: 0:00:00.0')
+        assert err.startswith('Elapsed: 0:00:00.0')
 
 
 def test_base_debug(base_app, capsys):
@@ -649,19 +719,19 @@ def test_edit_no_editor(base_app, capsys):
     base_app.editor = None
 
     # Make sure we get an exception, but cmd2 handles it
-    run_cmd(base_app, 'ed')
+    run_cmd(base_app, 'edit')
     out, err = capsys.readouterr()
 
     expected = _expected_no_editor_error()
     assert normalize(str(err)) == expected
 
-def test_edit_file(base_app, request):
+def test_edit_file(base_app, request, monkeypatch):
     # Set a fake editor just to make sure we have one.  We aren't really going to call it due to the mock
     base_app.editor = 'fooedit'
 
     # Mock out the os.system call so we don't actually open an editor
     m = mock.MagicMock(name='system')
-    os.system = m
+    monkeypatch.setattr("os.system", m)
 
     test_dir = os.path.dirname(request.module.__file__)
     filename = os.path.join(test_dir, 'script.txt')
@@ -669,15 +739,31 @@ def test_edit_file(base_app, request):
     run_cmd(base_app, 'edit {}'.format(filename))
 
     # We think we have an editor, so should expect a system call
-    m.assert_called_once_with('{} {}'.format(base_app.editor, filename))
+    m.assert_called_once_with('"{}" "{}"'.format(base_app.editor, filename))
 
-def test_edit_number(base_app):
+def test_edit_file_with_spaces(base_app, request, monkeypatch):
     # Set a fake editor just to make sure we have one.  We aren't really going to call it due to the mock
     base_app.editor = 'fooedit'
 
     # Mock out the os.system call so we don't actually open an editor
     m = mock.MagicMock(name='system')
-    os.system = m
+    monkeypatch.setattr("os.system", m)
+
+    test_dir = os.path.dirname(request.module.__file__)
+    filename = os.path.join(test_dir, 'my commands.txt')
+
+    run_cmd(base_app, 'edit "{}"'.format(filename))
+
+    # We think we have an editor, so should expect a system call
+    m.assert_called_once_with('"{}" "{}"'.format(base_app.editor, filename))
+
+def test_edit_number(base_app, monkeypatch):
+    # Set a fake editor just to make sure we have one.  We aren't really going to call it due to the mock
+    base_app.editor = 'fooedit'
+
+    # Mock out the os.system call so we don't actually open an editor
+    m = mock.MagicMock(name='system')
+    monkeypatch.setattr("os.system", m)
 
     # Run help command just so we have a command in history
     run_cmd(base_app, 'help')
@@ -687,13 +773,13 @@ def test_edit_number(base_app):
     # We have an editor, so should expect a system call
     m.assert_called_once()
 
-def test_edit_blank(base_app):
+def test_edit_blank(base_app, monkeypatch):
     # Set a fake editor just to make sure we have one.  We aren't really going to call it due to the mock
     base_app.editor = 'fooedit'
 
     # Mock out the os.system call so we don't actually open an editor
     m = mock.MagicMock(name='system')
-    os.system = m
+    monkeypatch.setattr("os.system", m)
 
     # Run help command just so we have a command in history
     run_cmd(base_app, 'help')
@@ -725,16 +811,16 @@ def test_base_cmdloop_with_queue():
     # Create a cmd2.Cmd() instance and make sure basic settings are like we want for test
     app = cmd2.Cmd()
     app.use_rawinput = True
-    app.intro = 'Hello World, this is an intro ...'
+    intro = 'Hello World, this is an intro ...'
     app.cmdqueue.append('quit\n')
     app.stdout = StdOut()
 
     # Need to patch sys.argv so cmd2 doesn't think it was called with arguments equal to the py.test args
     testargs = ["prog"]
-    expected = app.intro + '\n'
+    expected = intro + '\n'
     with mock.patch.object(sys, 'argv', testargs):
-        # Run the command loop
-        app.cmdloop()
+        # Run the command loop with custom intro
+        app.cmdloop(intro=intro)
     out = app.stdout.buffer
     assert out == expected
 
@@ -764,6 +850,7 @@ def test_cmdloop_without_rawinput():
     # Create a cmd2.Cmd() instance and make sure basic settings are like we want for test
     app = cmd2.Cmd()
     app.use_rawinput = False
+    app.echo = False
     app.intro = 'Hello World, this is an intro ...'
     app.stdout = StdOut()
 
@@ -773,7 +860,7 @@ def test_cmdloop_without_rawinput():
 
     # Need to patch sys.argv so cmd2 doesn't think it was called with arguments equal to the py.test args
     testargs = ["prog"]
-    expected = app.intro + '\n{}'.format(app.prompt)
+    expected = app.intro + '\n'
     with mock.patch.object(sys, 'argv', testargs):
         # Run the command loop
         app.cmdloop()
@@ -823,6 +910,29 @@ def test_default_to_shell_unknown(shell_app):
     out = run_cmd(shell_app, unknown_command)
     assert out == ["*** Unknown syntax: {}".format(unknown_command)]
 
+def test_default_to_shell_good(capsys):
+    app = cmd2.Cmd()
+    app.default_to_shell = True
+    if sys.platform.startswith('win'):
+        line = 'dir'
+    else:
+        line = 'ls'
+    statement = app.parser_manager.parsed(line)
+    retval = app.default(statement)
+    assert not retval
+    out, err = capsys.readouterr()
+    assert out == ''
+
+def test_default_to_shell_failure(capsys):
+    app = cmd2.Cmd()
+    app.default_to_shell = True
+    line = 'ls does_not_exist.xyz'
+    statement = app.parser_manager.parsed(line)
+    retval = app.default(statement)
+    assert not retval
+    out, err = capsys.readouterr()
+    assert out == "*** Unknown syntax: {}\n".format(line)
+
 
 def test_ansi_prompt_not_esacped(base_app):
     prompt = '(Cmd) '
@@ -1150,12 +1260,14 @@ def test_clipboard_failure(capsys):
 
 def test_run_command_with_empty_arg(base_app):
     command = 'help'
+    base_app.feedback_to_output = True
     run_cmd(base_app, command)
     out = run_cmd(base_app, 'run')
     expected = normalize('{}\n\n'.format(command) + BASE_HELP)
     assert out == expected
 
 def test_run_command_with_empty_history(base_app):
+    base_app.feedback_to_output = True
     out = run_cmd(base_app, 'run')
     assert out == []
 
@@ -1187,3 +1299,227 @@ def test_cmdresult(cmdresult_app):
     run_cmd(cmdresult_app, 'negative {}'.format(arg))
     assert not cmdresult_app._last_result
     assert cmdresult_app._last_result == cmd2.CmdResult('', arg)
+
+
+@pytest.fixture
+def abbrev_app():
+    app = cmd2.Cmd()
+    app.abbrev = True
+    app.stdout = StdOut()
+    return app
+
+def test_exclude_from_history(abbrev_app, monkeypatch):
+    # Run all variants of run
+    run_cmd(abbrev_app, 'run')
+    run_cmd(abbrev_app, 'ru')
+    run_cmd(abbrev_app, 'r')
+
+    # Mock out the os.system call so we don't actually open an editor
+    m = mock.MagicMock(name='system')
+    monkeypatch.setattr("os.system", m)
+
+    # Run all variants of edit
+    run_cmd(abbrev_app, 'edit')
+    run_cmd(abbrev_app, 'edi')
+    run_cmd(abbrev_app, 'ed')
+
+    # Run all variants of history
+    run_cmd(abbrev_app, 'history')
+    run_cmd(abbrev_app, 'histor')
+    run_cmd(abbrev_app, 'histo')
+    run_cmd(abbrev_app, 'hist')
+    run_cmd(abbrev_app, 'his')
+    run_cmd(abbrev_app, 'hi')
+
+    # Verify that the history is empty
+    out = run_cmd(abbrev_app, 'history')
+    assert out == []
+
+    # Now run a command which isn't excluded from the history
+    run_cmd(abbrev_app, 'help')
+    # And verify we have a history now ...
+    out = run_cmd(abbrev_app, 'history')
+    expected = normalize("""-------------------------[1]
+help""")
+    assert out == expected
+
+
+def test_is_text_file_bad_input(base_app):
+    # Test with a non-existent file
+    file_is_valid = base_app.is_text_file('does_not_exist.txt')
+    assert not file_is_valid
+
+    # Test with a directory
+    dir_is_valid = base_app.is_text_file('.')
+    assert not dir_is_valid
+
+
+def test_eof(base_app):
+    # Only thing to verify is that it returns True
+    assert base_app.do_eof('dont care')
+
+def test_eos(base_app):
+    sdir = 'dummy_dir'
+    base_app._script_dir.append(sdir)
+    assert len(base_app._script_dir) == 1
+
+    # Assert that it does NOT return true
+    assert not base_app.do_eos('dont care')
+
+    # And make sure it reduced the length of the script dir list
+    assert len(base_app._script_dir) == 0
+
+def test_echo(capsys):
+    app = cmd2.Cmd()
+    # Turn echo on and pre-stage some commands in the queue, simulating like we are in the middle of a script
+    app.echo = True
+    command = 'help history'
+    app.cmdqueue = [command, 'quit', 'eos']
+    app._script_dir.append('some_dir')
+
+    assert app._current_script_dir is not None
+
+    # Run the inner _cmdloop
+    app._cmdloop()
+
+    out, err = capsys.readouterr()
+
+    # Check the output
+    assert app.cmdqueue == []
+    assert app._current_script_dir is None
+    assert out.startswith('{}{}\n'.format(app.prompt, command) + 'history [arg]: lists past commands issued')
+
+def test_pseudo_raw_input_tty_rawinput_true():
+    # use context managers so original functions get put back when we are done
+    # we dont use decorators because we need m_input for the assertion
+    with mock.patch('sys.stdin.isatty',
+            mock.MagicMock(name='isatty', return_value=True)):
+        with mock.patch('six.moves.input',
+                mock.MagicMock(name='input', side_effect=['set', EOFError])) as m_input:
+            # run the cmdloop, which should pull input from our mocks
+            app = cmd2.Cmd()
+            app.use_rawinput = True
+            app._cmdloop()
+            # because we mocked the input() call, we won't get the prompt
+            # or the name of the command in the output, so we can't check
+            # if its there. We assume that if input got called twice, once
+            # for the 'set' command, and once for the 'quit' command,
+            # that the rest of it worked
+            assert m_input.call_count == 2
+    
+def test_pseudo_raw_input_tty_rawinput_false():
+    # gin up some input like it's coming from a tty
+    fakein = io.StringIO(u'{}'.format('set\n'))
+    mtty = mock.MagicMock(name='isatty', return_value=True)
+    fakein.isatty = mtty
+    mreadline = mock.MagicMock(name='readline', wraps=fakein.readline)
+    fakein.readline = mreadline
+    
+    # run the cmdloop, telling it where to get input from
+    app = cmd2.Cmd(stdin=fakein)
+    app.use_rawinput = False
+    app._cmdloop()
+    
+    # because we mocked the readline() call, we won't get the prompt
+    # or the name of the command in the output, so we can't check
+    # if its there. We assume that if readline() got called twice, once
+    # for the 'set' command, and once for the 'quit' command,
+    # that the rest of it worked
+    assert mreadline.call_count == 2
+
+# the next helper function and two tests check for piped
+# input when use_rawinput is True.
+def piped_rawinput_true(capsys, echo, command):
+    app = cmd2.Cmd()
+    app.use_rawinput = True
+    app.echo = echo
+    # run the cmdloop, which should pull input from our mock
+    app._cmdloop()
+    out, err = capsys.readouterr()
+    return (app, out)
+
+# using the decorator puts the original function at six.moves.input
+# back when this method returns
+@mock.patch('six.moves.input',
+        mock.MagicMock(name='input', side_effect=['set', EOFError]))
+def test_pseudo_raw_input_piped_rawinput_true_echo_true(capsys):
+    command = 'set'
+    app, out = piped_rawinput_true(capsys, True, command)
+    out = out.splitlines()
+    assert out[0] == '{}{}'.format(app.prompt, command)
+    assert out[1] == 'abbrev: False'
+
+# using the decorator puts the original function at six.moves.input
+# back when this method returns
+@mock.patch('six.moves.input',
+        mock.MagicMock(name='input', side_effect=['set', EOFError]))
+def test_pseudo_raw_input_piped_rawinput_true_echo_false(capsys):
+    command = 'set'
+    app, out = piped_rawinput_true(capsys, False, command)
+    firstline = out.splitlines()[0]
+    assert firstline == 'abbrev: False'
+    assert not '{}{}'.format(app.prompt, command) in out
+
+# the next helper function and two tests check for piped
+# input when use_rawinput=False
+def piped_rawinput_false(capsys, echo, command):
+    fakein = io.StringIO(u'{}'.format(command))
+    # run the cmdloop, telling it where to get input from
+    app = cmd2.Cmd(stdin=fakein)
+    app.use_rawinput = False
+    app.echo = echo
+    app.abbrev = False
+    app._cmdloop()
+    out, err = capsys.readouterr()
+    return (app, out)
+
+def test_pseudo_raw_input_piped_rawinput_false_echo_true(capsys):
+    command = 'set'
+    app, out = piped_rawinput_false(capsys, True, command)
+    out = out.splitlines()
+    assert out[0] == '{}{}'.format(app.prompt, command)
+    assert out[1] == 'abbrev: False'
+
+def test_pseudo_raw_input_piped_rawinput_false_echo_false(capsys):
+    command = 'set'
+    app, out = piped_rawinput_false(capsys, False, command)
+    firstline = out.splitlines()[0]
+    assert firstline == 'abbrev: False'
+    assert not '{}{}'.format(app.prompt, command) in out
+
+#
+# other input tests
+def test_raw_input(base_app):
+    base_app.use_raw_input = True
+    fake_input = 'quit'
+
+    # Mock out the input call so we don't actually wait for a user's response on stdin
+    m = mock.Mock(name='input', return_value=fake_input)
+    sm.input = m
+
+    line = base_app.pseudo_raw_input('(cmd2)')
+    assert line == fake_input
+
+def test_stdin_input():
+    app = cmd2.Cmd()
+    app.use_rawinput = False
+    fake_input = 'quit'
+
+    # Mock out the readline call so we don't actually read from stdin
+    m = mock.Mock(name='readline', return_value=fake_input)
+    app.stdin.readline = m
+
+    line = app.pseudo_raw_input('(cmd2)')
+    assert line == fake_input
+
+def test_empty_stdin_input():
+    app = cmd2.Cmd()
+    app.use_rawinput = False
+    fake_input = ''
+
+    # Mock out the readline call so we don't actually read from stdin
+    m = mock.Mock(name='readline', return_value=fake_input)
+    app.stdin.readline = m
+
+    line = app.pseudo_raw_input('(cmd2)')
+    assert line == 'eof'
